const fs = require('fs')
const { isdir, isfile, error, } = require('../common')

function extract(){
  const pwd = process.env.PWD
  const paths = [
    `${pwd}`,
    `${pwd}/conf`,
    `${pwd}/config`,
  ]

  let conf = null

  paths.map(p=>{
    const filename = `${p}/struct.json`
    if(isdir(p) && isfile(filename)){
      conf = require(filename)
    }
  })

  if(conf === null){
    error(' struct.json 文件无法找到')
    return false
  }

  return conf
}

function mkdirInObj(dir,dirs,cb,parent='/'){
  for(const pn in dirs){
    const current = `${dir}/${pn}`

    if(!isdir(current)){
      fs.mkdirSync(current)
    }
    if(dirs[pn]){
      mkdirInObj(current,dirs[pn],cb,pn)
    }
    cb && cb(pn,current,parent)
  }
}

function mkFile(key,dirPath,filename,constants,cb){
  const filePath = `${dirPath}/${filename}`
  if(!isfile(filePath)){
    fs.writeFile(filePath,'',{flag:'wx',mode:'0777'},()=>{
      cb && cb(key,filePath,filename,constants)
    })
  }
}

function matchPaths(proname,dir,dirs,paths,parent=''){
 for(const n in dirs){
   paths[`${proname}/${parent}${n}`] = `${dir}/${proname}/${parent}${n}/`
   if(typeof dirs[n] === 'object'){
     matchPaths(proname,dir,dirs[n],`${n}-`,paths)
   }
 }
}

function build(proname,conf,cb,dir=process.env.PWD){
  const dirs = conf['dirs']
  const files = conf['files']
  const rootFiles = conf['rootFiles']
  const rootProFiles = conf['rootProFiles']
  const configFiles = conf['configFiles']
  const proPath = `${dir}/${proname}`
  const collectDirPaths = {}

  if(!isdir(proPath)){
    fs.mkdirSync(proPath)
  }

  matchPaths(proname,proPath,dirs,collectDirPaths)

  const constants = {
    MATCH_PATHS: collectDirPaths,
    LOCAL_NODE_MODULES: `${process.env.pwd}/node_modules`,
  }

  rootFiles && rootFiles.map(filename=>{
    mkFile('/',dir,filename,constants,cb)
  })

  configFiles && configFiles.map(filename=>{
    mkFile('/config',`${dir}/config`,filename,constants,cb)
  })

  rootProFiles && rootProFiles.map(filename=>{
    mkFile('project',proPath,filename,constants,cb)
  })

  mkdirInObj(proPath,dirs,(pn,current,parent)=>{
    const key = `${parent === '/'?'':`${parent}/`}${pn}`
    const myfiles = files[key]

    for(const n in myfiles){
      const filename = myfiles[n]
      const filePath = `${current}/${filename}`

      mkFile(key,current,filename,constants,cb)
    }
  })
}

module.exports = {
  extract,
  build,
}
